/********************************************************************/
/*              Header for Camera extern module library functions   */
/********************************************************************/
#ifndef __CAMERA_H
#define __CAMERA_H

#include <stdio.h>  /* for sizeof, NULL, printf... */
#include <stdlib.h> /* for calloc, malloc, realloc, free... */

/* List of SFRs for Camera module */
/* This list contains the SFRs with default values to be used for configuring the module */
/* The user can modify this based on the requirement */

#if defined(__dsPIC33FJ64GP706__) || defined(__dsPIC33FJ64GP708__) ||defined(__dsPIC33FJ64GP710__) ||\
defined(__dsPIC33FJ128GP706__) ||defined(__dsPIC33FJ128GP708__) ||defined(__dsPIC33FJ128GP710__) ||\
defined(__dsPIC33FJ256GP710__) ||defined(__dsPIC33FJ128GP706PS__) ||defined(__dsPIC33FJ128GP708PS__) ||\
defined(__dsPIC33FJ256GP710PS__) ||defined(__dsPIC33FJ64MC706__) ||defined(__dsPIC33FJ64MC710__) ||\
defined(__dsPIC33FJ128MC706__) ||defined(__dsPIC33FJ128MC708__) ||defined(__dsPIC33FJ128MC710__) ||\
defined(__dsPIC33FJ256MC710__) ||defined(__dsPIC33FJ128MC706PS__) ||defined(__dsPIC33FJ128MC708PS__) ||\
defined(__dsPIC33FJ256MC710PS__) ||defined(__PIC24HJ256GP610__)

/* DEFAULT: */
#define CAMSTATUS_VALUE                 0x0000

//#define CAMCONFIG_VALUE                 0x124F // 80x64-jpeg  0x124F			SI
//#define CAMCONFIG_VALUE                 0x1A4F // 80x64-jpeg  0x1A4F send=1	SI
//#define CAMCONFIG_VALUE                 0x924E // 80x60-rgb16 0x924E			
//#define CAMCONFIG_VALUE                 0x924D // 80x60-rgb12 0x924D
//#define CAMCONFIG_VALUE                 0x924B // 80x60-Gray8 0x924B			SI
#define CAMCONFIG_VALUE                 0x1E4F // 80x64-jpeg  0x1E4F video	SI 44-45
//#define CAMCONFIG_VALUE                 0x.... // 160x128-jpeg  0x1E4F video	SI 22-23
//#define CAMCONFIG_VALUE                 0x.... // 320x240-jpeg  0x1E4F video	SI 8-9
//#define CAMCONFIG_VALUE                 0x9C4B // 80x60-Gray8 0x9C4B video	SI 16-17
//#define CAMCONFIG_VALUE                 0x9ADB // 160x120-Gray8 0x9ADB		NO
//#define CAMCONFIG_VALUE                 0x9CDB // 160x120-Gray8 0x9CDB video	NO
//#define CAMCONFIG_VALUE                 0x9C4A // 80x60-Gray4 0x9C4A video	SI 8-9
//#define CAMCONFIG_VALUE                 0x9CDA // 160x120-Gray4 0x9CDA video	SI 5-6
//#define CAMCONFIG_VALUE                 0x9C49 // 80x60-Gray2 0x9C49 video	SI 10-11
//#define CAMCONFIG_VALUE                 0x9CD9 // 160x120-Gray2 0x9CD9 video	SI 7-8
//#define CAMCONFIG_VALUE                 0x9C4D // 80x60-RGB12 0x9C4D video	SI 6-7 Colore???
//#define CAMCONFIG_VALUE                 0x9C4E // 80x60-RGB16 0x9C4E video	SI 5-6
//#define CAMCONFIG_VALUE                 0x7A4F // 115200 jpeg 80x64 snap
//#define CAMCONFIG_VALUE                 0xFA49 // 115200 2-bit gray 80x60 snap
//#define CAMCONFIG_VALUE                 0x7E4F // 115200 jpeg 80x64 video

#define CAMFRAMESIZE_VALUE              0x0200
#define CAMFRAMESIZE_DEFAULT            512
#define CAMDROPFRAMES_VALUE             0x0000

//#define CAMRXCOUNTER_VALUE              0x0000
//#define CAMTXCOUNTER_VALUE              0x0000
//#define CAMRBCOUNTER_VALUE              0x0000
//#define IMAGECOUNTER_VALUE              0x0000

//#define IMAGESIZE_VALUE                 0x00000000
//#define CAMSTOP_VALUE                   0x0000

#define VIDEOSIZE_VALUE                 500

/* CAMERA Configuration Bit Definitions */
// Command Constants
#define CAM_CMD_INITIAL		    0xAA01
#define CAM_CMD_GETPICTURE	    0xAA04
#define CAM_CMD_SNAPSHOT	    0xAA05
#define CAM_CMD_SETPACKETSIZE	0xAA06
#define CAM_CMD_SETBAUDRATE	    0xAA07
#define CAM_CMD_RESET		    0xAA08
#define CAM_CMD_POWEROFF	    0xAA09
#define CAM_CMD_DATA		    0xAA0A
#define CAM_CMD_SYNC		    0xAA0D
#define CAM_CMD_ACK		        0xAA0E
#define CAM_CMD_NAK		        0xAA0F
// #define CAM_CMD_LIGHTFREQUENCY	0xAA13

// NAK Error Constants
/*
#define CAM_ERR_PICTURE_TYPE		    0x01
#define CAM_ERR_PICTURE_UP_SCALE	    0x02
#define CAM_ERR_PICTURE_SCALE_ERROR	    0x03
#define CAM_ERR_UNEXPECTED_REPLY	    0x04
#define CAM_ERR_SEND_PICTURE_TIMEOUT	0x05
#define CAM_ERR_UNEXPECTED_COMMAND	    0x06
#define CAM_ERR_SRAM_JPEG_TYPE_ERROR	0x07
#define CAM_ERR_SRAM_JPEG_SIZE_ERROR	0x08
#define CAM_ERR_PICTURE_FORMAT_ERROR	0x09
#define CAM_ERR_PICTURE_SIZE_ERROR	    0x0a
#define CAM_ERR_PARAMETER_ERROR		    0x0b
#define CAM_ERR_SEND_REGISTER_TIMEOUT	0x0c
#define CAM_ERR_COMMAND_ID_ERROR	    0x0d
#define CAM_ERR_PICTURE_NOT_READY	    0x0f
#define CAM_ERR_TRANSFER_PACKAGE_NUMBER	0x10
#define CAM_ERR_TRANSFER_PACKAGE_SIZE	0x11
#define CAM_ERR_COMMAND_HEADER_ERROR	0xF0
#define CAM_ERR_COMMAND_LENGTH_ERROR	0xF1
#define CAM_ERR_SEND_PICTURE_ERROR	    0xF5
#define CAM_ERR_SEND_COMMAND_ERROR	    0xFF
*/
#define CAM_ERR_COMMAND_HEADER_ERROR	0xF0
#define IMAGE_ERROR 					0x0000
#define IMAGE_OK 						0x0001

#define CAM_ERR_SW_ERROR0	            0xE000 /* Reading error */
#define CAM_ERR_SW_ERROR1	            0xE100 /* Full queue */
#define CAM_ERR_SW_ERROR2	            0xE200 /* Packsize segmentation fault */
#define CAM_ERR_SW_ERROR3	            0xE300 /* No JPEG frame */
#define CAM_ERR_SW_ERROR4	            0xE400 /* Error in the final JPEG package */
#define CAM_ERR_SW_ERROR5	            0xE500 /* Over-Time error */
#define CAM_ERR_SW_ERROR6	            0xE600 /* Full queue (JPEG mode) */
#define CAM_ERR_SW_ERROR7	            0xE700 /* Command not expected */
#define CAM_ERR_SW_ERROR8	            0xE800 /* ACK error */
#define CAM_ERR_SW_ERROR9	            0xE900 /* DATA error */
#define CAM_ERR_SW_ERROR10	            0xEA00 /* JPEG test 1 error */
#define CAM_ERR_SW_ERROR11	            0xEB00 /* JPEG test 2 error */
#define CAM_ERR_SW_ERROR12	            0xEC00 /* JPEG test 3 error */
//#define CAM_ERR_SW_ERROR13	            0xED00 /* Queue error */



// CMD_INITIAL ColorType Constants (take only 3 bits)
#define COLORTYPE_BW_2		0x0001
#define COLORTYPE_BW_4		0x0002
#define COLORTYPE_BW_8		0x0003
#define COLORTYPE_COLOR_12	0x0005
#define COLORTYPE_COLOR_16	0x0006
#define COLORTYPE_JPEG		0x0007

// CMD_INITIAL Size Constants (take only 3 bits)
#define SIZE_80x64		    0x0001
#define SIZE_160x128		0x0003
#define SIZE_320x240		0x0005
#define SIZE_640x480		0x0007

// CMD_SETBAURATE BuadRate Constants (take only 3 bits)
#define BR7200			0x00FF
#define BR9600			0x00BF
#define BR14400			0x007F
#define BR19200			0x005F
#define BR28800			0x003F
#define BR38400			0x002F
#define BR57600			0x001F
#define BR115200		0x000F

// CMD_RESET ResetMode Costants
#define RESET_SYSTEM		0x0000
#define RESET_STATE_MACHINE	0x0010
#define SPECIAL_RESET	    0x00FF
#define SIMPLE_RESET	    0x0000

// CMD_SNAPSHOT Costants
#define COMPRESSED_PICTURE	 0x0000
#define UNCOMPRESSED_PICTURE 0x0001

// GENERIC Costants
#define COMPRESSED_VIDEO	 0x0000
#define UNCOMPRESSED_VIDEO   0x0001

// CMD_DATA DataType Constants (take only 2 bits)
#define DATATYPE_SNAPSHOT		0x0001
#define DATATYPE_PREVIEW		0x0002
#define DATATYPE_JPEG_PREVIEW	0x0003 // send -> 0x05
#define DATATYPE_JPEG_VIDEO		0x0005 // User can't use this costant

// CAMSTATUS FLAGS: (shift CAMSTATUS and take the proper number of bits)
#define SYNCHRO_FLAG	0x0000
#define READY_FLAG	    0x0001
#define WAIT_FLAG	    0x0002
#define ACK_FLAG	    0x0003
#define DATA_FLAG	    0x0004
#define NAK_FLAG	    0x0005
#define SYNC_FLAG	    0x0006
#define IMAGE_FLAG	    0x0007
#define ERROR_FLAG	    0x0008

#define SYNCHRO_NUM	    0x0001
#define READY_NUM	    0x0001
#define WAIT_NUM	    0x0001
#define ACK_NUM	        0x0001
#define DATA_NUM	    0x0001
#define NAK_NUM	        0x0001
#define SYNC_NUM	    0x0001
#define IMAGE_NUM	    0x0001
#define ERROR_NUM	    0x00FF

// CAMCONFIG FLAGS:
#define COLOR_FLAG	    0x0000
#define PREVRES_FLAG	0x0003
#define JPEGRES_FLAG	0x0006
#define PICTYPE_FLAG	0x0009
#define SEND_FLAG	    0x000B
#define BAUDDIV_FLAG	0x000C
#define SNAPTYPE_FLAG	0x000F

#define COLOR_NUM	    0x0007
#define PREVRES_NUM	    0x0007
#define JPEGRES_NUM	    0x0007
#define PICTYPE_NUM	    0x0003
#define SEND_NUM	    0x0001
#define BAUDDIV_NUM	    0x0007
#define SNAPTYPE_NUM	0x0001

// CAMFRAMESIZE FLAGS:
#define PACKSIZE_FLAG	0x0000
#define PACKSIZE_NUM	0x03FF

#define QF_SIZE	        21
#define IMAGE_SIZE	    10752 //512*21
//#define FRAME_SIZE	    520


typedef struct CAMERACOMMAND
{
	unsigned int  Id;			// Command ID
	unsigned long int Cam_Cmd; 
} CAMCMD;




/* FUNCTIONS: */

unsigned char Cam_InitialCmd(void);
unsigned char Cam_AckCmd(unsigned long int ackcmd);
unsigned char Cam_PackSizeCmd(void);
unsigned char Cam_SetBaudrateCmd(unsigned long int bdiv);
unsigned char Cam_GetCmd(void);
unsigned char Cam_ResetCmd(unsigned long int restype,unsigned long int spreset);
unsigned char Cam_PowerOffCmd(void);
unsigned char Cam_SnapshotCmd(void);
unsigned char Cam_SyncCmd(void);
void UART1_Config(unsigned int baudvalue);
void UART2_Config(unsigned int baudvalue);
void TIMER9_Config(unsigned long int match_value);
//void TIMER89_Config(unsigned long int match_value);
unsigned char Cam_ReadData(void);
unsigned char Cam_ReadCmd(void);
unsigned char Cam_ReadDataImage(void);
unsigned char Cam_SendCmd(CAMCMD *Hex_Cmd); 
void Send_by_UART2(unsigned int *buffer, unsigned char n);
void Cam_SendError(void);
void Send_by_UART1(unsigned int *buffer, unsigned char n);
void Cam_Init(void);
unsigned char Cam_GetStream(void);     
void Cam_Turnoff(void);
unsigned char Cam_GetPicture(unsigned char type);    
unsigned char Cam_GetVideo(unsigned char type);     
unsigned char Cam_JpegPicture(void);
unsigned char Cam_UncompressedPicture(void);
unsigned char Cam_JpegVideo(void);
unsigned char Cam_UncompressedVideo(void);
unsigned char Cam_Synchronization(float Fcy);    
float Cam_RoundFloat(float x2rx);
unsigned char Cam_Wait(unsigned int numf, unsigned int id,unsigned long int stbTime);
unsigned char Cam_Check(unsigned int numf, unsigned int id);
void Cam_SWError(unsigned int errtype);
unsigned char Cam_Wait2SendPicture(unsigned long int stbTime,unsigned char sendout);
unsigned char Cam_SendPicture(void);
void Cam_SendImage(void);
unsigned char Cam_ReadJPEGFrame(unsigned int ind);
unsigned char Cam_SendJPEGFrame(unsigned char send, unsigned int id);
void Cam_Sleep(unsigned long int xTime);
void Cam_CloseTimer(void);
void Cam_SendReceivedData(void);
void Cam_CloseTimer89(void);
void Cam_WriteTimer89( unsigned long int tval);
void Cam_OpenTimer89(unsigned int tconfig,unsigned long int tper);
void Cam_ConfigIntTimer89(unsigned int tconf);
void Cam_SendInfo(void);
void Cam_TIMER89Config(unsigned long int match_value);

extern volatile unsigned char FRAME[10000] __attribute__((far));
extern volatile unsigned int JPEGDIM;    
extern volatile unsigned int CAMCOUNTER;                        
extern volatile float CAMFCY;
extern volatile unsigned int CAMBUF[6];
extern volatile unsigned int CAMSTATUS;                    
extern volatile unsigned int CAMCONFIG;                   
extern volatile unsigned int CAMFRAMESIZE;              
extern volatile unsigned int CAMDROPFRAMES;            
extern volatile unsigned int CAMRBCOUNTER;              				
extern volatile unsigned long int IMAGECOUNTER;              			 
extern volatile unsigned long int IMAGESIZE;               				 
extern volatile unsigned char CAMSTOP;                       			 
extern volatile unsigned long int VIDEOSIZE;              
extern volatile long int BCOUNTER;              				 
extern volatile unsigned long int TXCOUNTER;              				 
extern volatile unsigned long int NEWLIMIT;              				 
extern volatile unsigned char SEND;
extern volatile unsigned char CTYPE;
extern volatile unsigned char start1; 
extern volatile unsigned char start2;
extern volatile unsigned char ust[15];
extern volatile unsigned char Buf[10];


#endif
#endif

