/***********************************************************************
 * Add the suitable linker script (e.g., p33FJ256MC710.gld )           *
 * to the project.                                                     *
 * If interrupts are not used, all code presented for that interrupt   *
 * can be removed or commented out with C-style comment declarations.  *
 * For additional information about dsPIC architecture and language    *
 * tools, refer to the following documents:                            *
 * MPLAB C30 Compiler User's Guide                        : DS51284    *
 * dsPIC 30F MPLAB ASM30, MPLAB LINK30 and Utilites                    *
 *                                           User's Guide : DS51317    *
 * Getting Started with dsPIC DSC Language Tools          : DS51316    *
 * dsPIC 30F Language Tools Quick Reference Card          : DS51322    *
 * dsPIC 30F 16-bit MCU Family Reference Manual           : DS70046    *
 * dsPIC 30F Motor Control and Power Conversion Family                 * 
 *                                             Data Sheet : DS70082    *
 * dsPIC 30F Programmer's Reference Manual                : DS70030    *
 * Template file has been compiled with MPLAB C30 v1.30.               *
 ***********************************************************************
 *    Author:            Dario Di Stefano                              *
 *    Company:           University of Pisa                            * 
 *    File Description:  CAMERA DEMO                                   *
 *    Date:              10/2007                                       *
 *    File Version:      1.0                                           *
 *    Other Files Required: p33FJ256MC710.gld                          *
 **********************************************************************/
 

#include "p33FJ256MC710.h"
#include <math.h>
#include "uart.h"
#include "timer.h"
#include "Camera.h"

   
/*************** Oscillator Configuration ****************/                                                                       
// Primary (XT, HS, EC) Oscillator with PLL
_FOSCSEL(FNOSC_PRIPLL);// & TEMP_OFF);
// OSC2 Pin Function: OSC2 is Clock Output - Primary Oscillator Mode: XT Crystanl
_FOSC(OSCIOFNC_ON & POSCMD_XT);
_FWDT( FWDTEN_OFF );




/************** Define constants here ***************/
	
#define Fcy 39e6 //non 40e6  /* dsPIC Frequency [Hz] -> almost 40 MIPS (40 MHz) */




/************** CAMERA EXTERN MODULE GLOBAL DEFINITIONS ***************/

volatile unsigned char FRAME[10000] __attribute__((far));
volatile unsigned int JPEGDIM=0;    
volatile unsigned int CAMCOUNTER=0;                        
volatile float CAMFCY = Fcy;
volatile unsigned int CAMBUF[6]={0,0,0,0,0,0};
volatile unsigned int CAMSTATUS = CAMSTATUS_VALUE;                    
volatile unsigned int CAMCONFIG = CAMCONFIG_VALUE;                   
volatile unsigned int CAMFRAMESIZE = CAMFRAMESIZE_VALUE;              
volatile unsigned int CAMDROPFRAMES = CAMDROPFRAMES_VALUE;            
volatile unsigned int CAMRBCOUNTER = 0;              				
volatile unsigned long int IMAGECOUNTER = 0;              			 
volatile unsigned long int IMAGESIZE = 0;               				 
volatile unsigned char CAMSTOP = 0;                       			 
volatile unsigned long int VIDEOSIZE = VIDEOSIZE_VALUE;              
volatile long int BCOUNTER = 0;              				 
volatile unsigned long int TXCOUNTER = 0;              				 
volatile unsigned long int NEWLIMIT = 0;              				 
volatile unsigned char SEND=0;
volatile unsigned char CTYPE=0;




/************* START OF GENERIC GLOBAL DEFINITIONS **********/

//Prototypes:
void SetupAutoBaud(void);        //Function to set up UART1, IC1 and TMR3
void CalculateBaud(void);        //Function to calculate the U1BRG value
//Variables:
unsigned int ICCount = 0;        //Count the number of Capture events
unsigned int T3Count = 0;        //Count the number of Timer 3 interrupts
unsigned long int CurrentCapture;     //Record time of UART edge
unsigned long int PreviousCapture;    //Store previous edge time measurement
unsigned long int CaptureDifference;  //Difference between times of UART edges
unsigned long int SumOfBitTimes;              //
//unsigned long BRate;          //Calculate the baud rate
volatile unsigned char start1 =0; 
volatile unsigned char start2 =0;
volatile unsigned char ust[15];
volatile unsigned char Buf[10];
unsigned char finish=0;
unsigned int U1BRG_expval, U1BRG_calcval;


/************** END OF GLOBAL DEFINITIONS ***********/



/************** MAIN ***********/

int main ( void )
{ 	
    unsigned char camok=0;
	unsigned int i=0;
    // Fcy/115200/16-1 = 5.78 -> 6
	/* Clock setup for 40MIPS */
    RCONbits.SWDTEN = 0;                    // Disable Watch Dog Timer
    CLKDIVbits.DOZEN   = 0;
    CLKDIVbits.PLLPRE  = 0;
    CLKDIVbits.PLLPOST = 0;
    PLLFBDbits.PLLDIV  = 76;				// to have Fcy = 39e6Hz...
    while(OSCCONbits.LOCK!=1);            	/* Wait for PLL to lock */
    
	//U1BRG=20;
	//U1BRG=20; // BAUD_RATE = 115200
	//U1BRG=41; // BAUD_RATE = 57600
	U1BRG=62; // BAUD_RATE = 38400
	//U1BRG=125; // BAUD_RATE = 19200
	//U1BRG=253; // BAUD_RATE = 9600
	UART1_Config(U1BRG);

  	//while(1)                                  //Loop forever
   	//{
		//Main routine
		//Loops forever detecting the baud rate from incoming UART data of 0x55
		//and outputing a message each time the baud rate is calculated.
    	//SetupAutoBaud();                       //Set up UART1, IC1 and TMR3 for autobaud
      	//while(U1BRG == 0) {}                   //Wait for autobaud to complete
      	//BRate = ((unsigned )Fcy / 16) / (U1BRG + 1);   //See what baud rate is being used
      	//printf("Baud rate: %ld\r", BRate);  //Output text with the baud rate
      	//while(finish == 0);
		//finish=0;
		
		//Cam_TIMER89Config(152343);
		//Cam_Sleep(76170);	//0.1sec
		
		//IEC0bits.IC1IE = 0;           	//Clear Capture 1 interrupt enable
        //IEC0bits.T3IE = 0;            	//Clear Timer 3 interrupt enable
		//U1BRG_calcval = (unsigned int)(((SumOfBitTimes + 64) >> 7) - 1);
		//UART1_Config( U1BRG_calcval );
		
		//Cam_Sleep(152343);
		//Cam_CloseTimer89();

//		ust[0] = (U1BRG>>8) & 0x00FF;
//		ust[1] = U1BRG & 0x00FF;
//		Send_by_UART1((unsigned int *)ust,2);
//		while(BusyUART1());		//Wait for transmission to complete
//		Send_by_UART1((unsigned int *)ust,2);
//		while(BusyUART1());		//Wait for transmission to complete
		//ust[0] = 0xAA;
		//ust[1] = 0xBB;
		//ust[2] = 0xCC;
		//ust[3] = 0xDD;
		//Send_by_UART1((unsigned int *)ust,4);
		//while(BusyUART1());		//Wait for transmission to complete
		//break;
   	//}

	//UART1_Config(259); 
	//ust[0]=0xAA;
	//Send_by_UART1((unsigned int *)ust,1);
	//while(BusyUART1());		//Wait for transmission to complete
	
	//UART1_Config(62); 
	//UART1_Config(20);         // Initialize the UART module to receive and transmit serial data.
    //UART1_Config(41);

	Cam_Init();
	
	//		while(1)
	//		{
	//			ust[0]=0xAA;
	//			Send_by_UART1((unsigned int *)ust,1);
	//			for(i=0;i<60000;i++);
	//		}
	//start1=1;

    while(1)
    {
        if(start1)
        {
			start1=0;
			camok = Cam_Synchronization( (float)Fcy );
			if(camok)
			{
				ust[0]=0x12;
				ust[1]=0x34;
				//Send_by_UART1((unsigned int *)ust,2);
			}
			else
			{
				ust[0]=0xEE;
				ust[1]=0xEE;
				Send_by_UART1((unsigned int *)ust,2);
				continue;
			}
			//if(!camok) continue;
			camok = Cam_GetStream();
		}
    }         
	//CloseUART1();
    //while(1);      
  	return 0;
}

// void SetupAutoBaud(void) -> Read me: Set up the peripherals and interrupts to do baud rate detection
void SetupAutoBaud(void)
{
   U1BRG = 0;                 //U1BRG initially unknown
   U1MODE = 0x8820;           //Enable auto baud detection in UART
   U1STA = 0x85D0;            //Set up rest of UART to default state with transmission enabled
 
   ICCount = 0;               //Initialize the number of Capture events
   IC1CON = 0x0000;           //Reset Input Capture 1 module
   IC1CON = 0x0002;           //Enable Input Capture 1 module
   IFS0bits.IC1IF = 0;        //Clear Capture 1 interrupt flag
   IEC0bits.IC1IE = 1;        //Enable Capture 1 interrupt

   T3CON = 0x0000;            //Timer 3 off
   IEC0bits.T3IE = 0;         //Clear Timer 3 interrupt enable
   T3Count = 0;               //Initialize the number of Timer 3 interrupts
   PR3 = 0xffff;              //Timer 3 period is maximum 
   T3CON = 0x8000;            //Timer 3 on with 1:1 prescaler and internal clock                
}

/****** START OF INTERRUPT SERVICE ROUTINES *********/
/* Replace the interrupt function names with the    */
/* appropriate names depending on interrupt source. */
/* The names of various interrupt functions for     */
/* each device are defined in the linker script.    */
/* Interrupt Service Routine 1                      */
/* No fast context save, and no variables stacked   */
/****************************************************/

/* This is: Input Capture 1 ISR */
/* Read me:
   	Gets time measurements and adds to the sum of the bit times
	Calculates U1BRG value after summing eight bit times  
*/
void _ISR _IC1Interrupt(void)
{
   IFS0bits.IC1IF = 0;                 //Clear Capture 1 interrupt flag
   PreviousCapture = CurrentCapture;   //Store previous time measurement 
   CurrentCapture = IC1BUF;            //Get new time measurement
   T3Count = 0;                        //Reset the timeout counter
   if(ICCount == 0)                    //Check if first edge
   {
      IFS0bits.T3IF = 0;               //Clear Timer 3 interrupt flag
      IEC0bits.T3IE = 1;               //Enable Timer 3 interrupt for timeout check
      SumOfBitTimes = 0;               //Initial value for sum of the bit times
   }  
   else                                //Check if not first edge
   {
      //if(ICCount != 9)                 //Check if not last edge
      if(ICCount <= 4)  
	  {
         CaptureDifference = CurrentCapture - PreviousCapture; //Get time difference
         SumOfBitTimes += CaptureDifference; //Add time difference to sum of times
		 if(ICCount == 4) finish=1;
      }
//      else                             //Check if last edge
//      {
//         IEC0bits.IC1IE = 0;           //Clear Capture 1 interrupt enable
//         IEC0bits.T3IE = 0;            //Clear Timer 3 interrupt enable
//		 //U1STA = 0x0000;			   // U1STA in default state		
//		 //U1MODE = 0x0000;			   // U1MODE disabled...
//         //U1BRG = ((SumOfBitTimes + 64) >> 7) - 1; //Calculate UxBRG (with rounding)
//         //U1MODE = 0x8800;              //Enable UART and disable auto baud detection
//         //U1STA = 0x85D0;               //Enable transmission
//		 UART1_Config(((SumOfBitTimes + 64) >> 7) - 1); 
//      }  
   }
   ICCount++;                          //Increment count of edges
}

/* This is: T3 ISR */
/* Read me:
   		Check for timeout indicated by two rollovers since previous input capture
*/
void _ISR _T3Interrupt(void)
{
   IFS0bits.T3IF = 0;            //Clear Timer 3 interrupt flag
   T3Count++;                    //Increment count of interrupts since last capture
   if(T3Count == 2)              //Check for too many timer rollovers since last capture 
   {
      SetupAutoBaud();           //Timeout so start over
   }
}

/* This is: T9 ISR */
/* Read me:
   This ISR can clear the wait time flag. 
   This time deadline helps the module to recognize
   the end of an image or a miss command. 
*/
void __attribute__((interrupt, no_auto_psv)) _T9Interrupt(void)
{
	IFS3bits.T9IF = 0;
	CAMSTATUS = CAMSTATUS & (~(WAIT_NUM<<WAIT_FLAG));     // Clear timer flag
}

/* This is: UART1 receive ISR */
/* Read me:
   This ISR can allow to communicate with the Camera
   extern module by dsPIC. 
   This interface has got a list of commands, described 
   below, to turn on/off the module, to configure the 
   structures or to start other functions.  
*/
void __attribute__((interrupt, no_auto_psv)) _U1RXInterrupt(void)
{
    unsigned long int x,y,z;
	IFS0bits.U1RXIF = 0;
    int i=0;
    for(i=0; i<4 && DataRdyUART1(); i++)
        Buf[i] = ReadUART1();  
    /* Prompt: AA 00 ** ** -> Start Camera module and get a picture or a video;
               AA CC ** ** -> Config the Camera module structures: Write CAMCONFIG;
               AA CF ** ** -> Config the Camera module structures: Write CAMFRAMESIZE;
               AA CD ** ** -> Config the Camera module structures: Write CAMDROPFRAMES;
               BB 00 00 00 -> Turn off the Camera module;
               CC .. .. .. -> Start Camera Calibration algorithm; 
               DD 00 xx xx -> Other;
               EE .. .. .. -> Get error byte;
               FF xx xx xx -> Other;  
	*/
    switch (Buf[0])
    {
			/*******************************/
    		case 0xAA: 
			     if(start2==0)
                 {   			
			         x = Buf[1];
				     y = Buf[2];
				     z = Buf[3];
				     
				     y = (y<<8) + z;
				     
                     if(x==0xCC)
				     {
                         CAMCONFIG = y;
                         //ust[0] = 0xCC;
                         ust[0] = (CAMCONFIG>>8) & 0x00FF;
				         ust[1] = CAMCONFIG & 0x00FF;
				         Send_by_UART1((unsigned int *)ust,2);
                     }
				     else if(x==0xCF)
				     {
                         CAMFRAMESIZE = y;
                         //ust[0] = 0xCF;
                         ust[0] = (CAMFRAMESIZE>>8) & 0x00FF;
				         ust[1] = CAMFRAMESIZE & 0x00FF;
				         Send_by_UART1((unsigned int *)ust,2);
                     }
                     else if(x==0xCD)
				     {
                         CAMDROPFRAMES = y;
                         //ust[0] = 0xCD;
                         ust[0] = (CAMDROPFRAMES>>8) & 0x00FF;
				         ust[1] = CAMDROPFRAMES & 0x00FF;
				         Send_by_UART1((unsigned int *)ust,2);
                     }
					 else if(x==0xBB)
				     {
                         U1BRG_expval = y;
                         if( (abs(U1BRG_expval-U1BRG_calcval)*20)<U1BRG_calcval )
						 {
                         	ust[0] = 0xBB;
				         	ust[1] = 0xBB;
				         }
						 else
						 {
                         	ust[0] = 0xEE;
				         	ust[1] = 0xEE;
				         }
						 Send_by_UART1((unsigned int *)ust,2);
                     }
                     else if(x==0x01)
                     {
				         start1=1; 
				         start2=0;
                     }
                     else if(x==0x02)
                     {
				         start1=0; 
				         start2=1;
                     }
                     else
                     {
                         ust[0] = (char)0xFF;
                         ust[1] = (char)0xFF;
				         ust[2] = (char)0xFF;
				         Send_by_UART1((unsigned int *)ust,3);
                     }	
                 }	
                 break;
            /*******************************/                 
			case 0xBB: // U1BRG: UART1 Baud rate value
				 start1=0;
				 start2=0;
				 //PowerOff_Cmd();
				 Cam_Turnoff();	
            	 break;
			/*******************************/
        	case 0xCC: 
				 start1=0;
				 start2=0;
				 //PowerOff_Cmd();
				 Cam_Turnoff();
				 break;  
      		/*******************************/
			case 0xDD:
				 start1=0;
				 start2=0;
				 //PowerOff_Cmd();
				 Cam_Turnoff();
                 break;
			/*******************************/	
			case 0xEE: 
                 start1=0;
				 start2=0;
				 //PowerOff_Cmd();
				 Cam_Turnoff();
                 break;
			/*******************************/
			case 0xFF: 
				 start1=0;
				 start2=0;
				 //PowerOff_Cmd();
				 Cam_Turnoff();
            	 break;
			/*******************************/
        	default:     
				if(start2==0);
//                {
//                	ust[0] = (char) Buf[0];
//                	ust[1] = (char) Buf[1];
//                	ust[2] = (char) Buf[2];
//                	ust[3] = (char) Buf[3];
//                	Send_by_UART1((unsigned int *)ust,4);
//            	}
			/*******************************/
	 }
}

/* This is: UART2 receive ISR */
/* Read me:
   This ISR can read the and recognize the camera commands
   or images.  
*/
void __attribute__((interrupt, no_auto_psv)) _U2RXInterrupt(void)
{
	IFS1bits.U2RXIF = 0;
    Cam_ReadData();
}

/*********************************************************THE END********************************************************************/

