/*
Copyright (C) 2007 - Nastasi Christian
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "flexCAM.h"
#include <iostream>
#include <signal.h>
using namespace std;
#define PROG_TAG "\nFlex-CAM Image Serial Reader :  "

FlexCAM::FlexCAM() : 	color_type(0), resolution_type(0), baud_rate_pc(0),  //baud_rate_flex(0),
			video_stream(false),  compression(false),
			error_code(0), width(0), height(false), channel(false), data_size(0),
			connected(false), init(false), config(false)
{
	buff1 = NULL;
	buff2 = NULL;
}

/***************************************************************/

bool 
FlexCAM::set_image_type(unsigned char type, unsigned char size, bool is_video)
{ 
	init=false;

	switch(type)
	{
	  case FLEX_CAM_GRAY2:
	  case FLEX_CAM_GRAY4:
	  case FLEX_CAM_GRAY8:
		channel=1;
		compression=false;
		break;

	  case FLEX_CAM_RGB12:
	  case FLEX_CAM_RGB16:
		channel=3;
		compression=false;
		break;
	  case FLEX_CAM_JPEG:
		compression=true;
		channel=3;
		break;

	  default:
		error_code = FLEX_ERR_BAD_COLOR_TYPE; 
		return false;
	}
	color_type=type;

	switch(size)
	{
	  case FLEX_CAM_SMALL:
		width=80;
		if(type==FLEX_CAM_JPEG) height=64;
		else height=60;
		break;
		
	  case FLEX_CAM_MED:
		width=160;
		if(type==FLEX_CAM_JPEG) height=128;
		else height=120;
		break;
		
	  case FLEX_CAM_LARGE:
		width=320;
		height=240;
		break;

	  default:
		error_code = FLEX_ERR_BAD_RES_TYPE; 
		return false;
	}
	resolution_type=size;

	video_stream = is_video;
	data_size = width*height*channel;

	init = true;
	return true; 
}

/***************************************************************/

bool
FlexCAM::connect(const char* serial_name)
{
	connected=false;

	if(!init){
		error_code = FLEX_ERR_NOT_INIT;
		return false;
	}

	if(buff1) delete[] buff1;
	if(buff2) delete[] buff2;
	buff1 = new unsigned char[width*height*channel];
	buff2 = new unsigned char[width*height*channel];
	memset(buff1, 0, width*height*channel);
	memset(buff2, 0, width*height*channel);

	if(!serial_name){
		if( !flex_Serial_init(&serial_port, FLEX_DEFAULT_SERIAL_PORT, baud_rate_pc) ){
			error_code = FLEX_ERR_SERIAL_OPEN; 
			return false;
		}
	}
	else{
		if( !flex_Serial_init(&serial_port, serial_name, baud_rate_pc) ){
			error_code = FLEX_ERR_SERIAL_OPEN; 
			return false;
		}
	}
	connected=true;
	return true;
}

/***************************************************************/

void
FlexCAM::close()
{
	flex_Serial_close(&serial_port); 
	if(buff1) delete[] buff1;
	if(buff2) delete[] buff2;
	buff1=NULL;
	buff2=NULL;
	connected=false;
	config=false;
}

/***************************************************************/

bool
FlexCAM::configure(const char* serial_name)
{
	unsigned char command[4] = {0xAA,0xCC,0,0};
	//unsigned char abrd_cmd[4] = {0x55,0,0,0};
	//unsigned char abrd_ack[4] = {0,0,0,0};
	//unsigned short int u1brg=0;
	unsigned char ack[2] = {0,0};
	unsigned short int cc=0;
	int status;

	config=false;

	if(!connected){
		error_code = FLEX_ERR_SERIAL_CLOSED;
		return false;
	}
	/*------------- Auto Baud rate detection: -------------*/
/*	
	cout << "Synchronizing PC-Flex serial communication ... ";
	fflush(stdout);
	Sleep(500);

	status = flex_Serial_write(&serial_port, (char*)abrd_cmd, 1);       // 0x55 -> 'U' sync command...
	if(status==-1) {
		error_code = FLEX_ERR_SERIAL_WRITE; 
		return false;
	}
	else if(status==0) {
		error_code = FLEX_ERR_SERIAL_TIMEOUT; 
		return false;
	}
	// Calculus:
    //u1brg = (((unsigned short int)(abrd_ack[0]))<<8) & 0xFF00 + ((unsigned short int)(abrd_ack[1])) & 0x00FF;
//	u1brg = (unsigned short int)( 39000000.0/((double)(baud_rate_pc))/16.0 )-1 ;
//	abrd_cmd[0]= 0xAA;
//	abrd_cmd[1]= 0xBB;
//	abrd_cmd[2]= (u1brg>>8) & 0x00FF;
//	abrd_cmd[3]= u1brg & 0x00FF;
	
	this->close();
	Sleep(1000);
	
	if( !(this->connect(serial_name)) ){
		cout << PROG_TAG << "Connection Error : ErrorCode=" << this->get_error() << endl;
		return false;
	}
	Sleep(100);
	
//	status = flex_Serial_write(&serial_port, (char*)abrd_cmd, 4);       // AA BB U1BRGH U1BRGL
//	if(status==-1) {
//		error_code = FLEX_ERR_SERIAL_WRITE; 
//		return false;
//	}
//	else if(status==0) {
//		error_code = FLEX_ERR_SERIAL_TIMEOUT; 
//		return false;
//	}

	status = flex_Serial_read(&serial_port, (char*)abrd_ack, 4);
	if(status==-1) {
		error_code = FLEX_ERR_SERIAL_READ; 
		return false;
	}
	else if(status==0) {
		error_code = FLEX_ERR_SERIAL_TIMEOUT; 
		return false;
	}
    // checking:
    //cout << endl << abrd_ack[0]<< " " << abrd_ack[1]<< " " << abrd_ack[2]<< " " << abrd_ack[3] << endl;
    if(abrd_ack[0] != 0xAA || abrd_ack[1] != 0xBB || abrd_ack[2] != 0xCC || abrd_ack[3] != 0xDD){
		error_code = FLEX_ERR_BAD_ACK; 
		return false;
	}
	cout << "Done!";
*/
    /*------------------------------------------------------*/
	cout << "Loading image settings ... ";
	fflush(stdout);
	Sleep(500);
	cc = color_type;

	cc += ((unsigned short int) resolution_type) << 3;
	cc += ((unsigned short int) resolution_type) << 6;

	if(!video_stream) cc += ((unsigned short int) (1)) << 9;	// Snap-shot
	else if(!compression) cc += ((unsigned short int) (2)) << 9;	// Video-uncompressed
	else cc += ((unsigned short int) (3)) << 9;			// Video-compressed

	cc += ((unsigned short int) (1)) << 11;				// 1=not store in memory, 0=store in memory	

	//cc += ((unsigned short int) (1)) << 12;				//TODO: make this a  variable!!     1 = 9600 bps
    	cc += ((unsigned short int) (7)) << 12;				//TODO: make this a  variable!!     7 = 115200 bps
	if(!compression) cc += ((unsigned short int) (1)) << 15;	// Uncompressed format

	command[3] = cc & 0x00FF;		// CCL
	command[2] = (cc >> 8) & 0x00FF;	// CCH

	status = flex_Serial_write(&serial_port, (char*)command, 4);
	if(status==-1) {
		error_code = FLEX_ERR_SERIAL_WRITE; 
		return false;
	}
	else if(status==0) {
		error_code = FLEX_ERR_SERIAL_TIMEOUT; 
		return false;
	}

	status = flex_Serial_read(&serial_port, (char*)ack, 2);
	if(status==-1) {
		error_code = FLEX_ERR_SERIAL_READ; 
		return false;
	}
	else if(status==0) {
		error_code = FLEX_ERR_SERIAL_TIMEOUT; 
		return false;
	}

	if(command[3] != ack[1] || command[2] != ack[0]){
		error_code = FLEX_ERR_BAD_ACK; 
		return false;
	}

	config=true;
	return true;
}

/***************************************************************/

bool
FlexCAM::get_image(unsigned char* &image)
{
	unsigned char command[4] = {0xAA,0x01,0,0};
	int s=0;
	unsigned long int byte_count=0;
	unsigned char byte;
	bool is_marker=false;
	bool is_eoi=false;
	bool is_soi=false;

	if(!config){
		error_code = FLEX_ERR_NOT_CONFIG;
		return false;
	}

	if(buff1) memset(buff1, 0, width*height*channel);
	if(buff2) memset(buff2, 0, width*height*channel);

	s = flex_Serial_write(&serial_port, (char*)command, 4);
	if(s==-1) {
		error_code = FLEX_ERR_SERIAL_WRITE; 
		return false;
	}
	else if(s==0) {
		error_code = FLEX_ERR_SERIAL_TIMEOUT; 
		return false;
	}


	if(compression){
		while(1)
		{
			s = flex_Serial_getc(&serial_port,(char*)&byte);
			if(s==-1) {
				error_code = FLEX_ERR_SERIAL_READ; 
				return false;
			}
			else if(s==0) {
				error_code = FLEX_ERR_SERIAL_TIMEOUT; 
				return false;
			}

			if(!is_soi)
			{
				if(byte_count == 0 && byte==0xFF){
					is_marker = true;
					buff1[byte_count++] = byte;
					continue;
				}else if(is_marker && byte == 0xD8){
					is_soi = true;
					is_marker = false;
					buff1[byte_count++] = byte;
					continue;
				}else if(is_marker && byte!= 0xFF){
					byte_count=0;
					is_marker=false;
					continue;
				}
			}
			else{
				buff1[byte_count++] = byte;
				if(is_marker && byte==0xD9) is_eoi=true;	// Find EOI
				else is_marker=false;
				if(byte==0xFF) is_marker = true;
				if(is_eoi && (byte_count % 4 == 0)) break;
			}
		}
	}
	else{

		s = flex_Serial_read(&serial_port, (char*)buff1, data_size);
		if(s==-1) {
			error_code = FLEX_ERR_SERIAL_READ; 
			return false;
		}
		else if(s==0) {
			error_code = FLEX_ERR_SERIAL_TIMEOUT; 
			return false;
		}
	}


	/* NOW... building output image in buff2... */

	switch (color_type)
	{
		case FLEX_CAM_GRAY2 : 
			flex_gray2_to_gray8(buff1, buff2, width,height);
			image = buff2;
			break;
		case FLEX_CAM_GRAY4 : 
			flex_gray4_to_gray8(buff1, buff2, width, height);
			image = buff2;
			break;
		case FLEX_CAM_GRAY8: 
			image = buff1;
			break;
		case FLEX_CAM_RGB12 : 
			flex_rgb12_to_rgb24(buff1, buff2, width, height);
			image = buff2;
			break;
		case FLEX_CAM_RGB16 : 
			flex_rgb16_to_rgb24(buff1, buff2, width, height);
			image = buff2;
			break;
		case FLEX_CAM_JPEG : 
			int w,h,bpp;
			if(!flex_JPEG_decompress(buff1, byte_count, buff2, w, h, bpp)){
				error_code = FLEX_ERR_JPEG_DECOMPRESS; 
				return false;
			}
			image = buff2;
			break;
	}

	return true;
}

/***************************************************************/



